"use strict";
// Define error type and shared error checking / message building logic.
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqlPlaceholderError = void 0;
class SqlPlaceholderError extends Error {
    constructor(message, undefinedNames, unusedNames) {
        super(message);
        this.message = message;
        this.undefinedNames = undefinedNames;
        this.unusedNames = unusedNames;
    }
    static create(undefinedNames, unusedNames) {
        let ok = true;
        let messages = [];
        if (undefinedNames.length !== 0) {
            ok = false;
            messages.push(`SQL placeholders '${undefinedNames.join("', '")}' are used in SQL statement but value not given.`);
        }
        if (unusedNames.length !== 0) {
            ok = false;
            messages.push(`SQL placeholders '${unusedNames.join("', '")}' are given but didn't appear in SQL statement.`);
        }
        return ok ? null : new SqlPlaceholderError(messages.join(" "), undefinedNames, unusedNames);
    }
    static checkExhaustivity(placeholders, usedNames) {
        const definedNames = Object.keys(placeholders);
        usedNames = [...new Set(usedNames)]; // remove duplicated items
        const undefinedNames = usedNames.filter(name => !definedNames.includes(name));
        const unusedNames = definedNames.filter(name => !usedNames.includes(name));
        return SqlPlaceholderError.create(undefinedNames, unusedNames);
    }
}
exports.SqlPlaceholderError = SqlPlaceholderError;
//# sourceMappingURL=error.js.map