"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySqlPlaceholderResolver = void 0;
const core_1 = require("./core");
const error_1 = require("./error");
// Name of positional placeholder. MySQL now doesn't support "$i" syntax.
const POSITIONAL_PLACEHOLDER = "?";
class MySqlPlaceholderResolver {
    constructor(placeholders) {
        this.placeholders = placeholders;
        this.usedNames = [];
        this.values = [];
    }
    allocScalar(value) {
        this.values.push(value);
        return POSITIONAL_PLACEHOLDER;
    }
    allocArray(values) {
        if (values.length === 0) {
            return core_1.EMPTY_LIST_SQL;
        }
        // Build (?, ?, ?, ...).
        const text = values.map(v => this.allocScalar(v)).join(", ");
        return `(${text})`;
    }
    resolvePlaceholder(name) {
        this.usedNames.push(name);
        const value = this.placeholders[name];
        if (value instanceof Array) {
            return this.allocArray(value);
        }
        return this.allocScalar(value);
    }
    static resolve(statement, placeholders) {
        const resolver = new MySqlPlaceholderResolver(placeholders);
        const replacedStatement = statement.replace(core_1.PLACEHOLDER_REGEXP, (_text, name) => resolver.resolvePlaceholder(name));
        const err = error_1.SqlPlaceholderError.checkExhaustivity(placeholders, resolver.usedNames);
        if (err != null) {
            throw err;
        }
        return [replacedStatement, resolver.values];
    }
}
exports.MySqlPlaceholderResolver = MySqlPlaceholderResolver;
//# sourceMappingURL=resolver_mysql.js.map